# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Nicholas Skaggs <nicholas.skaggs@canonical.com>
#              Robert Steckroth, Surgemcgee Bust0ut <RobertSteckroth@gmail.com>

from testtools.matchers import Equals, NotEquals, Not, Is
from autopilot.matchers import Eventually
import time


class ubuntusdk(object):
    """An emulator class that makes it easy to interact with the ubuntu sdk applications."""

    def __init__(self, autopilot, app):
        self.app = app
        self.autopilot = autopilot

    def get_qml_view(self):
        """Get the main QML view"""
        return self.app.select_single("QQuickView")

    def switch_to_tab(self, tab):
        """Switch to the specified tab number"""
        tabs = self.app.select_single("Tabs", objectName="rootTabs")
        currentTab = tabs.selectedTabIndex
      
        #perform operations until tab == currentTab
        while tab != currentTab:
            if tab < currentTab:
               self._previous_tab()
            if tab > currentTab:
               self._next_tab()
            currentTab = tabs.selectedTabIndex

    def click_object(self, obj):
        self.autopilot.pointing_device.click_object(obj)

    def move_subtabs_bar_up(self):
        """Moves the subtabs bar to a different location"""
        qmlView = self.get_qml_view()
        sub_bar = self.app.select_single("QQuickRectangle", objectName='subTabsBar')
        button_width = sub_bar.select_single('UbuntuShape', objectName='detailsTab').width
        self.autopilot.pointing_device.move_to_object(sub_bar)

        self.autopilot.pointing_device.drag( self.autopilot.pointing_device.x-button_width-5, self.autopilot.pointing_device.y,
                                             self.autopilot.pointing_device.x-button_width-5, qmlView.height*0.3)


    def flick_page_up(self, obj):    
        """Moves the pointer around the charts sub tab"""
        startX = obj.x + 2
        stopX = obj.x + obj.width - 2
        lineY = obj.y + obj.height - 2
        self.autopilot.pointing_device.drag(startX, lineY, stopX/1.5, obj.y+2)

    def hold_page_down(self, obj):    
        """Grabs and holds the provided object downwards"""

        self.autopilot.pointing_device.move_to_object(obj)
        self.autopilot.pointing_device.press()
        self.autopilot.pointing_device.move(self.autopilot.pointing_device.x, obj.y+obj.height+60)

    def release_mouse_press(self):
        """Releases the mouse pointer"""
        self.autopilot.pointing_device.release()


    def flick_page_right(self, obj):    
        """Drag the pointer to the right of the provided object"""
        startX = obj.x + 40 # add the chart left padding
        stopX = obj.width - 2
        lineY = obj.y + obj.height - 2
        self.autopilot.pointing_device.drag(startX, lineY, stopX, lineY/1.5)

    def after_program_fully_loaded(self):
        """Returns when all of the applications ActivityIndicator.running state is False."""

        add_activity_indicator = self.app.select_single('ActivityIndicator', objectName='addActivityIndicator')        
        current_activity_indicator = self.app.select_single('ActivityIndicator', objectName='currentActivityIndicator')        
        charts_activity_indicator = self.app.select_single('ActivityIndicator', objectName='chartsActivityIndicator')        
        news_activity_indicator = self.app.select_single('ActivityIndicator', objectName='newsActivityIndicator')        
        self.autopilot.assertThat(add_activity_indicator.running, Eventually(Equals(False)))
        self.autopilot.assertThat(current_activity_indicator.running, Eventually(Equals(False)))
        self.autopilot.assertThat(charts_activity_indicator.running, Eventually(Equals(False)))
        self.autopilot.assertThat(news_activity_indicator.running, Eventually(Equals(False)))


    def open_toolbar(self):
        """Open the toolbar"""
        qmlView = self.get_qml_view()

        lineX = qmlView.x + qmlView.width * 0.50
        startY = qmlView.y + qmlView.height - 1
        stopY = qmlView.y + qmlView.height * 0.95

        self.autopilot.pointing_device.drag(lineX, startY, lineX, stopY)

    def hide_toolbar(self):
        """Hide the toolbar"""
        qmlView = self.get_qml_view()

        lineX = qmlView.x + qmlView.width * 0.50
        startY = qmlView.y + qmlView.height * 0.95
        stopY = qmlView.y + qmlView.height - 1

        self.autopilot.pointing_device.drag(lineX, startY, lineX, stopY)

    def _previous_tab(self):
        """Switch to the previous tab"""
        qmlView = self.get_qml_view()

        startX = qmlView.x + 20 
        stopX = qmlView.width * .7 
        lineY = qmlView.y + 20 

        self.autopilot.pointing_device.drag(startX, lineY, stopX, lineY)
        self.autopilot.pointing_device.move(stopX-80, lineY) # move back into the previous tab
        self.autopilot.pointing_device.click()
        self.autopilot.pointing_device.click()
        time.sleep(1) # give the app time to move the title text into position

    def _next_tab(self):
        """Switch to the next tab"""
        qmlView = self.get_qml_view()

        startX = qmlView.x + qmlView.width * 0.90
        stopX = qmlView.x + 20
        lineY = qmlView.y + 20

        self.autopilot.pointing_device.drag(startX, lineY, stopX, lineY)
        self.autopilot.pointing_device.move(stopX+20, lineY)
        self.autopilot.pointing_device.click()
        self.autopilot.pointing_device.click()
        time.sleep(1) # give the app time to move the title text into position
        




